program conred38;
{$F+}
{
***********************************************************
"Puente de tiristores con carga LER"
Autores: JRomero - Arodri
Fecha: 18-04-98

		 La simulacin corresponde a un puente de tiristores
		 con carga LER , y se obeservarn los siguientes
		 parmetros:

		 * tensines de entrada (todas las de  linea), contrastada con la de salida
		  tomando como carga una fuente de corriente.
		 * corriente de salida.

		 Se podr afectar el ngulo de disparo  y
		 el valor del mismo estar acotado entre 70 y 90 grados. El
		 recorrido se har a pasos de 0.2 grados.

		 La fuente E tendra un valor de  100 V.

		 El valor de la inductancia es 200mH.

		 OBS: para implementar el disparo por el control arcocoseno
		 se toma como referencia a la fase R y se supone que esta es
		 Vp sin(w*t).
**********************************************************
}

uses
wincrt, xmatdefs, FilUSR, UCntrl, Simul4, traxp, variable,
lib_basi, libep02, libep, libdig01, memorias,dibus;

const w=PI*100;           {frecuencia de red=50 hz}
		sexrad=PI/180;      {constante de conversin de sexagesimal a rad}
		periodo=20e-3;      {periodo=2*PI/w}
		grillay=8;          {divisiones de la grilla en y}
		grillax=24;			  {divisiones de la grilla en x}
		nper=2;				  {nmero de periodos a visualizar   (nper>=1)}
		NumeroPuntos=512;		{puntos de muestreo}
		alpha0=78.4;
		Ecarga=100;

var   alpha,alphamin,alphamax,wt:NReal;

		{ variables definidas para muestrear UD e id}
		UD,id,
		tMuestra,
		NMuestra:Nreal;
		mem1,mem2:Tmemoria;

(*********************************************************************
  Funcin para el disparo del tiristor.
*********************************************************************)
{
	El parmetro ALPHA es tomado como control del disparo del tiristor.
}

function disparo(k:integer): boolean;
var vref:NREAL;
begin
	vref:=-cos(alpha*sexrad);
	disparo:=(vref <= sin(w*t - 2*PI/3 - (k-1) * PI/3)) and (cos(w*t - 2*PI/3 - (k-1) * PI/3)>=0);
{	DISPARO:=TRUE;}
end;


procedure reponera0;far;
begin
	alpha:=alpha0;
end;



(***********************************************************************
	Funciones de definicin del circuito

	Cirt1: componentes no booleanos
	Cirt2: componentes booleanos
***********************************************************************)

procedure cirt1;
begin
vg(1,1,4,311);							{generador R de 220 Vrms}
vg(2,1,5,311);							{generador T de 220 Vrms}
vg(3,1,6,311);							{generador S de 220 Vrms}


vg(0,0,3,Ecarga);
l(2,3,200e-3);
r(2,3,10000);
r(7,2,0.001);

end;

procedure cirt2;
begin
th(4,7,disparo(1));							{Tiristor 1}
th(0,6,disparo(2));							{Tiristor 2}
th(5,7,disparo(3));							{Tiristor 3}
th(0,4,disparo(4));							{Tiristor 4}
th(6,7,disparo(5));							{Tiristor 5}
th(0,5,disparo(6));							{Tiristor 6}
end;


(************************************************************************
	Definicin de la seal del generador VG[1]
************************************************************************)
procedure funcsr(t:NReal);
begin
rf[1]:=sin(w*t);				{R}
rf[2]:=sin(w*t-2*PI/3);    {S}
rf[3]:=sin(w*t-4*PI/3);    {T}
end;




(************************************************************************
	Procedimientos necesarios de Simeep:

	inicio:	inicializacin de simulaciones, def. de pantalla,etc
	pesca: 	captura e impresin en pantalla de variables observadas
	fin:     procedimiento de finalizacin

************************************************************************)

procedure inicio;
begin
	nn:=7;					{nmero de nodos, sin incluir tierra (0)}
	nindep:=3;     		{nmero de funciones definidas en funcsr = numero de generadores}
	tinicial:=0;			{tiempo inicial de la simulacin}
	tfinal:=nper*periodo;	{tiempo final de la simulacin}


	{parmetros de simulacin}
	dtmax0:=5e-6;
	dtmin:=dtmax0/100;
	PasoDivisor:=6;
	TmaxTmin:=1e3;

	{corriente de carga inicial}
	x0[1]:=4;

	{inicializacin de la unidad de grficos}
	traxp.tinicial:=tinicial;
	traxp.tfinal:=tfinal;

	{inicializacin del modo grfico}
	iniciegr;

	{grillas en pantalla}
	gridx:=grillax;
	gridy:=grillay;

	{matriz de visualizacin de 1 por 4}
	subplot(1,2);


	{primera pantalla}
	definaY(0,-300,100);
	titulo('uD(t) (amarillo) y tensiones de linea: +-RS(rojo),+-ST(negro),+-TR(azul)');
	xlabel('30/DIV');
	ylabel('100 V/DIV');
	linea0;
	grid;

	{segunda pantalla}
	definaY(1,0,1);
	titulo('');
	xlabel('30 /DIV');
	ylabel('1 A/DIV');
	linea0;
	grid;

	{canal 4,5,6 en la misma pantalla que el canal 0}
	superponga(2,0);
	superponga(3,0);
	superponga(4,0);
	superponga(5,0);
	superponga(6,0);
	superponga(7,0);
	superponga(8,0);


	{colores a mostrar por canal}
	{primera pantalla}
	definacolor(0,marron);	{uD}
	definacolor(2,rojo);			{RS}
	definacolor(3,negro);      {ST}
	definacolor(4,azul);       {TR}
	definacolor(5,rojo);			{SR}
	definacolor(6,negro);     	{TS}
	definacolor(7,azul);      	{RT}
	definacolor(8,amarillo);      	{uD}

	{segunda pantalla}
	definacolor(1,azul);			{}

end;

{____________________________________________________________________}

procedure fin;
begin
TermineGr;
halt;
end;

{____________________________________________________________________}

procedure pesca;

var   k,enx:integer;
		grados:NReal;
begin

	traxp.t:=t;					{indicamos a traxp el tiempo de simulacin}
	wt:=w/sexrad*t;

	{impresion de etiquetas en pantalla}
	grados:=(w/sexrad)*t;
	enX:=PUNTOX+21; {paso la ordenada x que pinta la onda a la variable enX por medio de la funcion puntox}
	{ANOTAR_EVENTO(numeracin, GRAD1,GRAD2,GRAD, ENX, y,y1a,y1b,y2a,y2b,etiqueta)}
	{alpha}ANOTAR_EVENTO(1,30,30+alpha,grados,enx,30,20,220,20,220, CHR(224)+' para T1');



	{captura de valores}

	{primera pantalla}
	trazo(2,v[4]-v[5]);			{RS}
	trazo(3,v[5]-v[6]);      {ST}
	trazo(4,v[6]-v[4]);       {TR}
	trazo(5,v[5]-v[4]);			{-SR}
	trazo(6,v[6]-v[5]);      {-TS}
	trazo(7,v[4]-v[6]);       {-RT}
	trazo(0,v[7]);			{uD}
	trazo(0,Ecarga);			{E}
	trazo(8,v[7]+10);       {uD}
	trazo(8,v[7]-10);       {uD}


	{segunda pantalla}
	trazo(1,x[1]);		{}


	{pesca de UD para muestreo}
	if tMuestra>=periodo/6 then
	begin
		tMuestra:=0;
		NMuestra:=NMuestra+1;

		mem2.fft;
		id:=mem2.ValorMedio;
		mem2.retrocesobarrido(periodo/6);

		mem1.fft;
		ud:=mem1.ValorMedio;
		mem1.retrocesobarrido(periodo/6);
	end
	else tMuestra:=t-NMuestra*periodo/6;
	mem1.tic(tMuestra,v[7]);
	mem2.tic(tMuestra,x[1]);


	if t >= tfinal then
	begin
	{llegamos al final de la simulacin}

		{para el muestreo de UD}
		tMuestra:=0;
		NMuestra:=0;

		for k:= 0 to 8 do
		begin
			borrecanal(k,0);
			desligue(k);
			borde;
			grid;
			linea0;
		end;

		{controlar si la corriente no tomo valores desorbitados}

		if id>9 then
		begin

			poner_ventana(100,100,400,180,
			'   Se ha detectado que iD(t) ha alcanzado un',
			'valor exagerado para mostrarlo en pantalla.',
			'La  simulacin  se reiniciar,  pulsando el',
			'botn derecho del ratn.');

			alpha:=alpha0;
			{corriente de carga inicial}
			x0[1]:=4;
			x[1]:=4;
		end;

		{nueva simulacin}
		t:= t-tfinal+tinicial;


	end;



end;


(*********************************************************************
 Principal
*********************************************************************)
BEGIN
	quita:=5;			{separaciones de pantallas}

	alphamin:=70;
	alphamax:=90;

	{para el muestreo de UD e id}
	UD:=100.4;	{valor "inicial", antes de hacer el primer calculo}
	id:=4.1;
	NMuestra:=0;
	tMuestra:=0;
	mem1:= TMemoria.Create_init(periodo/6/128,128,0,0);	{ud}
	mem2:= TMemoria.Create_init(periodo/6/128,128,0,0);	{id}

	{decimos a simeep donde estan nuestras funciones}
	FilUSR.Cirt1:=Cirt1;
	FilUSR.Cirt2:=Cirt2;
	FilUSR.FuncsR:=FuncsR;
	FilUSR.Inicio:=Inicio;
	FilUSR.Pesca:=Pesca;
	FilUSR.Fin:=Fin;

	{definicin de controles}
	UCntrl.init;

	UCntrl.insertdisplay(new(PCtrlVerTexto, init
	{ rnglon, texto1  ,            texto2,             texto3}
	(	0,	'Puente de tiristores','con carga L-E-R','(L=200mh;E=100V)')));


	UCntrl.insert(new(PCtrlBotonVar2,Init
	{renglon, nombre,   unidad,decimales, min, max,   def, inc,    var}
	(  8,   ' Disparo',  ''  ,1    ,   alphamin, alphamax,   alpha0,  0.2,  alpha)));


	UCntrl.insert(new(PCtrlBotonProc,Init
	{renglon, nombre, proc}
	(  9,   '    Reponer '+chr(224)+'0', reponera0)));

	UCntrl.insertdisplay(new(PCtrlVer2Var,init
	{ renglon, nombre1, uni1,deci1,var1 ,nombre2, uni2,deci2,var2}
	(       6, '  <Ud>',  'V',  0  ,ud ,'  <iD>', 'A',  1  ,id)));
	UCntrl.insertdisplay(new(PCtrlVer2Var,init
	{ renglon, nombre1, uni1,deci1,var1 ,nombre2, uni2,deci2,var2}
	(       7, '   iD',  'A',  1  ,x[1] ,' grados', '',  1  ,wt)));

	paseo;												  {simulacin}

	mem1.Free;
	mem2.Free;

end.
